//! Implementation of a koreader progress sync server.

use anyhow::Result;
use api::Api;
use app_state::AppState;
use cli::Config;
use poem::{
    http::{uri::Scheme, Uri},
    listener::TcpListener,
    middleware::Tracing,
    EndpointExt, Route,
};
use poem_openapi::OpenApiService;
use std::sync::Arc;

pub mod api;
pub mod app_state;
pub mod auth_middleware;
pub mod cli;
pub mod db;
pub mod error_response;

/// Run the progress sync server.
pub async fn run(args: &Config, db_url: &str) -> Result<()> {
    let db = db::connect(db_url).await?;
    let app_state = Arc::new(AppState {
        config: args.clone(),
        db,
    });

    const API_PATH: &str = "/api";
    let api_uri = Uri::builder()
        .scheme(Scheme::HTTP)
        .authority(args.address.clone())
        .path_and_query(API_PATH)
        .build()?;

    let api_service = OpenApiService::new(Api, "Hesinde Sync", "1.0").server(api_uri.to_string());
    let ui = api_service.swagger_ui();
    let app = Route::new()
        .nest(API_PATH, api_service)
        .nest("/", ui)
        .data(app_state)
        .with(Tracing);

    Ok(poem::Server::new(TcpListener::bind(&args.address))
        .run(app)
        .await?)
}
